package com.app.blackjack;

import com.app.blackjack.cards.Deck;
import com.app.blackjack.players.Player;

import java.util.Arrays;

public class BlackJack {

    public static final int WINNING_WORTH = 21;

    private final Player[] players;

    private final Deck deck;

    public BlackJack(Player[] players, Deck deck) {
        this.players = players;
        this.deck = deck;
    }

    public void round() {
        deck.shuffle();
        drawCards();
        handleWinners();
        returnCards();
    }

    private void drawCards() {
        for (Player player : players)
            while (!player.isLosing() && player.wantCard() && deck.canDraw())
                player.receiveCard(deck.draw());
    }

    private void handleWinners() {
        int winnerWorth = getWinnerWorth();
        Arrays.stream(players)
                .filter(player -> player.getCurrentWorth() == winnerWorth)
                .forEach(Player::winRound);
    }

    private int getWinnerWorth() {
        return Arrays.stream(players)
                .filter(player -> !player.isLosing())
                .mapToInt(Player::getCurrentWorth)
                .max()
                .orElse(-1);
    }

    private void returnCards() {
        for (Player player : players)
            deck.addCards(player.returnCards());
    }
}
