package com.app.blackjack;

import com.app.blackjack.cards.Deck;
import com.app.blackjack.players.Player;
import com.app.blackjack.players.PlayerMapper;
import com.app.users.User;
import com.app.users.UserDatabase;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.IntStream;

import static java.util.stream.Collectors.toMap;

@Service
@AllArgsConstructor
public class BlackJackService {

    private final UserDatabase userDatabase;

    private final PlayerMapper mapper;

    public void playBlackJack(List<String> users, int rounds) {
        Map<User, Player> players = users.stream()
                .map(userDatabase::selectUser)
                .collect(toMap(Function.identity(), mapper::map));

        BlackJack blackJack = new BlackJack(players.values().toArray(new Player[0]), Deck.normal());

        IntStream.range(0, rounds).forEach(i -> blackJack.round());

        players.entrySet().stream()
                .map(e -> User.userWithNewWins(e.getKey(), e.getValue().getWinCount()))
                .forEach(userDatabase::insertIntoUsers);
    }
}
