package com.app.blackjack.cards;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Stream;

public class Deck {

    private Card[] cards;

    public static Deck empty() {
        return new Deck(new Card[0]);
    }

    public static Deck normal() {
        return new Deck(createNormalDeck());
    }

    public static Deck multiple(int multiplier) {
        if (multiplier <= 0)
            return empty();
        Card[][] decks = new Card[multiplier][];
        for (int i = 0; i < multiplier; i++) {
            decks[i] = createNormalDeck();
        }
        Card[] cards = Stream.of(decks)
                .map(Arrays::asList)
                .flatMap(List::stream)
                .toArray(Card[]::new);
        return new Deck(cards);
    }

    static Card[] createNormalDeck() {
        Card[] cards = new Card[Color.values().length * Rank.values().length];
        int i = 0;
        for (Color color : Color.values())
            for (Rank rank : Rank.values())
                cards[i++] = new Card(color, rank);
        return cards;
    }

    private Deck(Card[] cards) {
        this.cards = cards;
    }

    public void shuffle() {
        List<Card> cardsList = Arrays.asList(cards);
        Collections.shuffle(cardsList);
        cards = cardsList.toArray(new Card[0]);
    }

    public boolean canDraw() {
        return cards.length > 0;
    }

    public Card draw() {
        Card drawn = cards[cards.length - 1];
        cards = Arrays.copyOf(cards, cards.length - 1);
        return drawn;
    }

    public void addCards(Card[] newCards) {
        cards = Stream.concat(Arrays.stream(cards), Arrays.stream(newCards)).toArray(Card[]::new);
    }
}
