package com.app.blackjack;

import com.app.blackjack.cards.Card;
import com.app.blackjack.cards.Color;
import com.app.blackjack.cards.Deck;
import com.app.blackjack.cards.Rank;
import com.app.blackjack.players.Player;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.mockito.Mockito.*;

class BlackJackTest {

    private Deck deck;
    private Player player1, player2;

    private BlackJack blackJack;

    @BeforeEach
    void setUp() {
        deck = mock(Deck.class);
        player1 = mock(Player.class);
        player2 = mock(Player.class);
    }

    @Test
    void shouldNotDrawIfThereAreNoCards() {
        // given
        when(deck.canDraw()).thenReturn(false);
        when(deck.draw()).thenThrow(IndexOutOfBoundsException.class);

        Card[] returnedCards = new Card[0];
        when(player1.isLosing()).thenReturn(false);
        when(player1.wantCard()).thenReturn(true);
        when(player1.returnCards()).thenReturn(returnedCards);

        blackJack = new BlackJack(new Player[]{player1}, deck);

        // when
        blackJack.round();

        // then
        verify(deck, times(1)).shuffle();
        verify(deck, times(1)).canDraw();
        verify(deck, times(1)).addCards(returnedCards); // this check is too implementation-dependent
        verifyNoMoreInteractions(deck);

//        These checks are too implementation-dependent (probably)
        verify(player1, times(2)).isLosing();
        verify(player1, times(1)).wantCard();
        verify(player1, times(2)).getCurrentWorth();
        verify(player1, times(1)).winRound();
        verify(player1, times(1)).returnCards();
        verifyNoMoreInteractions(player1);
    }

    @Test
    void shouldWinIfHaveMoreWorth() {
        // given
        when(deck.canDraw()).thenReturn(true, true);
        when(deck.draw()).thenReturn(new Card(Color.CLUBS, Rank.ACE), new Card(Color.DIAMONDS, Rank.JACK));

        when(player1.isLosing()).thenReturn(false);
        when(player1.wantCard()).thenReturn(true, false);
        when(player1.getCurrentWorth()).thenReturn(Rank.ACE.worth);

        when(player2.isLosing()).thenReturn(false);
        when(player2.wantCard()).thenReturn(true, false);
        when(player2.getCurrentWorth()).thenReturn(Rank.JACK.worth);

        blackJack = new BlackJack(new Player[]{player1, player2}, deck);

        // when
        blackJack.round();

        // then
        verify(deck, times(1)).shuffle();
        verify(deck, times(2)).canDraw();
        verify(deck, times(2)).draw();

        verify(player1, times(1)).winRound();
        verify(player2, times(0)).winRound();
    }
}
